/* @(#) $Revision: 12.1 $ */

/*
   Remote operation is controlled by 2 defines:

   REMOTE_ONLY must be defined if compiling with remote only library.
   NO_REMOTE must be defined if compiling with Eloquence A.03.xx
*/

#ifndef ELOQDB_INCLUDED
#define ELOQDB_INCLUDED

#if !defined(__hpux)

/*
   Local mode is only supported on HP-UX platform
*/
#define REMOTE_ONLY

#if !defined(__cplusplus)
/*
   K&R Compiler is only supported on HP-UX.
   Some compilers are quite cautious about defining __STDC__. 
   If language extensions are enabled they still have all the goodies of
   ANSI-C but refrain from defining __STDC__ since it's no longer
   fully ANSI-compliant.
*/

#if !defined(__STDC__)
#define __STDC__
#endif
#endif	/* !__cplusplus */
#endif	/* !hpux */


#if defined(__cplusplus)
extern "C" {
#endif

/*------------------------------------------------------------------------
      idb limits
------------------------------------------------------------------------*/

#define MAX_MPASSWD       16     /* Max. length of m.password  */
#define MAX_UCL_CNT       32     /* No of security classes     */
#define MAX_PASSWD         8     /* Max. length of password    */
#define MAX_ITEM_CNT    1024     /* Max. number of items       */
#define MAX_ITEM_NAME     16     /* Max. length of item names  */
#define MAX_SET_CNT       99     /* Max. number of sets        */
#define MAX_SET_NAME      16     /* Max. length of set names   */
#define MAX_PATH_CNT      16     /* Max. number of pathes      */


/*------------------------------------------------------------------------
      idb defaults
------------------------------------------------------------------------*/

#define DEF_MAX_DB   1
#define DEF_MAX_PATH 16
#define DEF_MAX_BUF  4096
#define DEF_MAX_OPEN (DEF_MAX_PATH+1)


/*------------------------------------------------------------------------
      idb info response
------------------------------------------------------------------------*/

struct info_list {         /* 103, 104, 203, 204 */
   int count;              /* list entries   */
   int no[1];              /* list data      */
};

struct info_102 {
   char name[16];          /* item name      */
   char type;              /* item type      */
   char filler[3];         /* gap (space)    */
   int size;               /* item size      */
   int count;              /* subitem count  */
   int fmt;                /* format number  */
};

struct info_202 {
   char name[16];          /* set name       */
   char type;              /* set type       */
   char filler[3];         /* gap (space)    */
   int  rec_len;           /* record length  */
   long entries;           /* set entries    */
   long capacity;          /* set capacity   */
};

struct info_301 {
   int count;           /* list entries   */
   struct {
      int set;          /* related set    */
      int ditem;        /* detail item no */
   } path[1];
};

struct info_302 {
   int item;            /* item no        */
   int set;             /* related set/0  */
};

struct info_502 {
   char name[16];        /* iitem name     */
   int  seg_cnt;         /* segment cnt    */
   struct {
      int item;          /* item no        */
      int length;        /* seg length     */
   } seg[8];
};

union info
{
   int no;              /* used by modes 101,201 */
   struct info_list list;
   struct info_102 item;
   struct info_202 set;
   struct info_301 path_list;
   struct info_302 search_item;
   char volume[8];      /* unused */
   struct info_502 iitem;
};


/*------------------------------------------------------------------------
      dbml function id's
------------------------------------------------------------------------*/

#define DB_OPEN    401
#define DB_INFO    402
#define DB_CLOSE   403
#define DB_FIND    404
#define DB_GET     405
#define DB_UPDATE  406
#define DB_PUT     407
#define DB_DELETE  408
#define DB_LOCK    409
#define DB_UNLOCK  410

#define DB_CREATE  411
#define DB_ERASE   412
#define DB_PURGE   413


/*------------------------------------------------------------------------
      idb error codes
------------------------------------------------------------------------*/

#define S_OK        0      /* No error */
#define S_CANTOPEN -1      /* Unable to open database */
#define S_INUSE    -2      /* Database in use */
#define S_NOSLOT   -10     /* No empty database slot */
#define S_BADID    -11     /* Bad database id */
#define S_NOLOCK   -12     /* Conflicting lock request on write attempt */
#define S_READONLY -14     /* Database opened in readonly mode */
#define S_BAD      -21     /* Bad parameter value */
#define S_NOWRITE  -23     /* No write access to data set */
#define S_BADAUTO  -24     /* Bad write attempt on automatic master */
#define S_BADMODE  -31     /* Illegal mode */
#define S_BADITEM  -52     /* Item not an accessible seach item */
#define S_BADLIST  -52     /* Bad list variable */
#define S_VERSION  -91     /* Incompatible database version */
#define S_NOCREAT  -92     /* Database requires creation */
#define S_CORRUPT  -94     /* Database may be corrupt */
#define S_NOMASTER -95     /* No master set entry for current detail */
#define S_POINTER  -96     /* Corrupt pointer value detected */

#define S_NOMEM    -120    /* Not enough memory to perform DBLOCK */
#define S_LCKM122  -122    /* Descriptor list bad */
#define S_LCKM123  -123    /* Illegal relational operator */
#define S_LCKM124  -124    /* Descriptor too short */
#define S_LCKM125  -125    /* Bad set name/number */
#define S_LCKM126  -126    /* Bad item name/numner */
#define S_LCKM127  -127    /* Attempt to lock using a compound item */
#define S_LCKM128  -128    /* Bad descriptor length for numeric item */
#define S_LCKM134  -134    /* Two descriptors conflict */
#define S_LCKM135  -135    /* Second lock not with wait option */
#define S_LCKM136  -136    /* descriptor length exceeds maximum */
#define S_LCKM137  -137    /* qualifier parameter of wrong type */

#define S_EOF       11     /* End of file reached */
#define S_DIRBOF    12     /* Directed beginning of file */
#define S_DIREOF    13     /* Directed end of file */
#define S_ENDCHAIN  15     /* End of chain reached */
#define S_NOREC     17     /* No current record or record/key not found */
#define S_BROKEN    18     /* Broken chain (only get!) */

#define S_LCK20     20     /* Database locked or contains locks */
#define S_LCK22     22     /* Dataset locked */
#define S_LCK23     23     /* Entries locked within set */
#define S_LCK24     24     /* Item conflicts with current locks */
#define S_LCK25     25     /* Entry or entries already locked */
#define S_LCKOP     27     /* Relational operator type conflict */

#define S_UPDATE    41     /* Cant alter search item */
#define S_DUPL      43     /* Duplicate key value */
#define S_CANTDEL   44     /* Can't delete master with non-empty chains */
#define S_ARGTYPE   53     /* Bad argument type */
#define S_NOTFOUND  80     /* Set not created/not online */
#define S_COROPEN   94     /* Corrupt data base opened in mode 8 */
#define S_NOHEAD    100    /* No chain head for path xx */

#define S_ISAM      -800   /* Internal isam error */
#define S_DAEMON    -900   /* Daemon error */


/*------------------------------------------------------------------------
   Remote access supplies it's own status codes
   If a call failes, S_REMOTE will be returned in status[0] and
   status[9] will either contain errno (if above zero) or one of the
   codes below.
------------------------------------------------------------------------*/

#if !defined(NO_REMOTE)

#define S_REMOTE -700      /* remote problem */

#define S_CONNECT   -2     /* unable to connect server */
#define S_INIT      -3     /* init failed */
#define S_ARGOVFL   -4     /* client arg buffer overflow */
#define S_SEND      -5     /* problem during send */
#define S_RECEIVE   -6     /* problem during receive */
#define S_ROOT      -7     /* inconsistent ROOT structures detected */
#define S_BUFFER    -8     /* inconsistent buffer size detected */

#endif   /* NO_REMOTE */


/*------------------------------------------------------------------------
      externals   !! READONLY !!
------------------------------------------------------------------------*/

extern int idb_max_db;        /* Max. of data bases */
extern int idb_max_path;      /* Max. set links */
extern int idb_max_buf;       /* Internal buffer size + links */
extern int idb_max_open;      /* Max open sets */

extern int idb_status;        /* Status of last idb call */
extern int net_status;        /* Status of daemon communication */
extern int isam_status;       /* Status of last isam call */


/*------------------------------------------------------------------------
      prototypes
------------------------------------------------------------------------*/

#if defined(__STDC__) || defined(__cplusplus)
#define _p(a) a
#else
#define _p(a) ()
#endif

/* 
   compatiblity stuff for WIN16 API
*/

#if defined(__hpux)
#define EXPORT
#define FAR
#define PASCAL
#else
 #if defined(WINDOWS)
  #if !defined(WIN32)
   #define EXPORT __export
   #define FAR    _far
   #define PASCAL _pascal
  #else
   #define EXPORT
   #define FAR
   #define PASCAL
  #endif
 #else
  #define EXPORT
  #define FAR
  #define PASCAL
 #endif
#endif


/*
   StatusType used to be int on HP-UX and long on WIN16 platform.
   In order to preserve compatibility to existing source code, you
   may simply define StatusType as an integer.
*/

#if !defined(StatusType)
#ifdef __hpux
#define StatusType int
#else
#define StatusType long
#endif
#endif

/*
   idb function prototypes
*/

int FAR PASCAL idb_init _p((int, int, int, int));
int FAR PASCAL idb_exit _p((void));
int FAR PASCAL 
   idb_open _p((const char FAR *, const char FAR *, int, StatusType FAR[]));
int FAR PASCAL 
   idb_close _p((int, const void FAR *, int, StatusType FAR[]));
int FAR PASCAL 
   idb_info _p((int, const void FAR *, int, StatusType FAR[], void FAR *));
int FAR PASCAL 
   idb_lock _p((int, const void FAR *, int, StatusType FAR[]));
int FAR PASCAL 
   idb_unlock _p((int, const void FAR *, int, StatusType FAR[]));
int FAR PASCAL 
   idb_find _p((int, const void FAR *, int, StatusType FAR[], 
		const void FAR *, const void FAR *));
int FAR PASCAL 
   idb_get _p((int, const void FAR *, int, StatusType FAR[], 
	       const void FAR *, char FAR *, const void FAR *));
int FAR PASCAL 
   idb_put _p((int, const void FAR *, int, StatusType FAR[], 
	       const void FAR *, const char FAR *));
int FAR PASCAL 
   idb_update _p((int, const void FAR *, int, StatusType FAR[], 
		  const void FAR *, const char FAR *));
int FAR PASCAL 
   idb_delete _p((int, const void FAR *, int, StatusType FAR[]));
int FAR PASCAL 
   idb_error _p((StatusType FAR[], void FAR *, int FAR *));

/* idb_burst_mode() may not be used externally ! */
int FAR PASCAL idb_burst_mode _p((int, int, int, const int FAR *));


/*
   remote idb function prototypes
*/

#if !defined(NO_REMOTE)

#if defined(REMOTE_ONLY)
 int FAR PASCAL ridb_init _p((int));
#else
 int FAR PASCAL ridb_init _p((void));
#endif
int FAR PASCAL ridb_exit _p((void));
int FAR PASCAL
   ridb_open _p((const char FAR *, const char FAR *, int, StatusType FAR[]));
int FAR PASCAL
   ridb_close _p((int, const void FAR *, int, StatusType FAR[]));
int FAR PASCAL
   ridb_info _p((int, const void FAR *, int, StatusType FAR[], void FAR *));
int FAR PASCAL
   ridb_lock _p((int, const void FAR *, int, StatusType FAR[]));
int FAR PASCAL
   ridb_unlock _p((int, const void FAR *, int, StatusType FAR[]));
int FAR PASCAL
   ridb_find _p((int, const void FAR *, int, StatusType FAR[], 
		 const void FAR *, const void FAR *));
int FAR PASCAL
   ridb_get _p((int, const void FAR *, int, StatusType FAR[], 
		const void FAR *, char FAR *, const void FAR *));
int FAR PASCAL
   ridb_put _p((int, const void FAR *, int, StatusType FAR[], 
		const void FAR *, const char FAR *));
int FAR PASCAL
   ridb_update _p((int, const void FAR *, int, StatusType FAR[], 
		   const void FAR *, const char FAR *));
int FAR PASCAL
   ridb_delete _p((int, const void FAR *, int, StatusType FAR[]));

#endif   /* NO_REMOTE */

#undef _p


/*------------------------------------------------------------------------
   This section will redirect idb_... function calls to
   corresponding ridb_... routines.
   REMOTE_ONLY should be defined, if library only supports
   remote operation (no local idb library).
------------------------------------------------------------------------*/

#if defined(REMOTE_ONLY)

#define idb_init( a,b,c,d )         ridb_init(a)
#define idb_exit()                  ridb_exit()
#define idb_info( a,b,c,d,e )       ridb_info( a,b,c,d,e )
#define idb_open( a,b,c,d )         ridb_open( a,b,c,d )
#define idb_close( a,b,c,d )        ridb_close( a,b,c,d )
#define idb_find( a,b,c,d,e,f )     ridb_find( a,b,c,d,e,f )
#define idb_get( a,b,c,d,e,f,g )    ridb_get( a,b,c,d,e,f,g )
#define idb_update( a,b,c,d,e,f )   ridb_update( a,b,c,d,e,f )
#define idb_delete( a,b,c,d )       ridb_delete( a,b,c,d )
#define idb_put( a,b,c,d,e,f )      ridb_put( a,b,c,d,e,f )
#define idb_lock( a,b,c,d )         ridb_lock( a,b,c,d )
#define idb_unlock( a,b,c,d )       ridb_unlock( a,b,c,d )

#endif   /* REMOTE_ONLY */

#if defined(__cplusplus)
}
#endif

#endif   /* ELOQDB_INCLUDED */
