/* ----------------------------------------------------------------------
   (C) COPYRIGHT HEWLETT-PACKARD GmbH 1994
   
   Project:  HP ELOQUENCE
   Module:   dlg / dm
   File:     dlgapi.h
   Function: public dlg
   $Header: dlgapi.h,v 1.1 93/09/02 20:10:21 mike Exp $

   NOTE: 
      <windows.h> must be included before
      If subsequent include will cause duplicate types and #defines

   Edit History:
   24.10.93 - creation
   20.01.94 - merge of hpux / msw version
   06.02.94 - calling convention, fixed CDECL
   10.02.94 - dm_dlg_load( PATH )
   28.03.94 - dm_dlg_load()
  
---------------------------------------------------------------------- */

#ifndef DLGAPI_INCLUDED
#define DLGAPI_INCLUDED


/* ------------------------------------------------------------------------

   API types

------------------------------------------------------------------------ */

#ifndef DLGAPI_TYPES
#define DLGAPI_TYPES

#ifndef DOS

# define FAR    /**/
# define PASCAL /**/
# define CDECL  /**/

typedef char *LPSTR;
typedef const char *LPCSTR;

#else    /* DOS */

#ifndef _INC_WINDOWS

/* <windows.h> must be included before */

# define FAR    __far
# define PASCAL __pascal
# define CDECL  __cdecl

typedef char FAR *LPSTR;
typedef const char FAR *LPCSTR;

#endif   /* _INC_WINDOWS */

#endif   /* DOS */
#endif   /* DLGAPI_TYPES */


/* 
   DLG value container 
   ValueType MUST be combatible with eloquence definition
   or the interface code will fail.
*/

enum ValueType {
   DLG_VOID,      /* no value */
   DLG_STRING,    /* string value */
   DLG_INT        /* integer (32 bit, signed) value */
};

typedef struct Value {
   enum ValueType type;
   union {
      LPCSTR sv;
      long iv;
   } u;
} VALUE;



/* ------------------------------------------------------------------------

   API prototypes

------------------------------------------------------------------------ */

extern int FAR PASCAL dm_dlg_new(LPCSTR path, LPCSTR classname);
extern int FAR PASCAL dm_dlg_del(LPCSTR path);
extern int FAR PASCAL dm_dlg_get(LPCSTR path, VALUE FAR *val);
extern int FAR PASCAL dm_dlg_set(LPCSTR path, const VALUE FAR *val);
extern int FAR PASCAL dm_dlg_load(LPCSTR path,LPCSTR dlg,LPCSTR idm,LPCSTR idc,LPCSTR fname);
extern int FAR PASCAL dm_dlg_draw(LPCSTR path);
extern int FAR PASCAL dm_dlg_do(LPCSTR path, int options);
extern int FAR PASCAL dm_dlg_stop(void);
/* extern int FAR PASCAL dm_do_dlg_load(LPCSTR fname, LPCSTR *p_msg, int FAR *p_line_no); */
extern int FAR PASCAL dm_dlg_rule(LPCSTR name, LPCSTR path,
                       int argc, const VALUE FAR *argv, VALUE FAR *rval);
extern int FAR PASCAL dm_dlg_func( LPCSTR name, int argc, const VALUE FAR *argv, VALUE FAR *rval );
extern int FAR PASCAL dm_dlg_popup( int style, int btn_style, LPCSTR parent,
                         int x, int y, LPCSTR title, LPCSTR text,
                         int btn_cnt, LPCSTR btn[], int def_btn, int FAR *rbtn);
extern int FAR PASCAL dm_dlg_help( LPCSTR tag );

/*
   The following prototypes are only visible if IDMuser.h has been included 
*/

#ifdef __DMUSER_INCLUDED__

extern void FAR PASCAL dm_bind_default_callbacks( DM_ID dlg_id );
extern void FAR PASCAL set_active_dialog( DM_ID dlgID );

extern void DML_default DM_ENTRY ExitEventLoop(DM_ID obj, long rule);

extern int FAR CDECL dm_error_handler(LPCSTR file, int line, int code, LPCSTR msg, ...);
extern long FAR PASCAL dm_errno(void);

#endif


/*
   this functions must be provided by envirnment
*/

extern void FAR CDECL dmdrv_debug(int level, LPCSTR fmt, ...);
extern int  FAR PASCAL dmdrv_main_loop(long rule, LPCSTR path);

#ifdef __DMUSER_INCLUDED__

extern int FAR PASCAL dmdrv_attach_dialog(DM_ID objID, long value);
#ifdef DOS
extern DM_ID LookupIDMWindow(DM_ID objID);
#endif
extern boolean DML_default DM_ENTRY HelpOnObject(DM_ID objID);

#endif


/* ------------------------------------------------------------------------

   dlg return codes

------------------------------------------------------------------------ */

enum DialogReturnCodes {
   DLG_OK,           /* success, MUST be zero */
   DLG_FAILED,       /* internal failure */
   DLG_NOFILE,       /* unable to open file */
   DLG_SYNTAX,       /* syntax error in dlg file */
   DLG_NOSPC,        /* no space left in dialog table or MAX_DEPTH exceeded */
   DLG_NOMEM,        /* memory allocation failed */
   DLG_DUPL,         /* duplicate dialog or object name */
   DLG_BADPATH,      /* illegal or bad object path */
   DLG_BADATTR,      /* bad or improper attribute */
   DLG_BADCHILD,     /* parent object does not agree to child */
   DLG_BADTYPE,      /* bad or improper value type */
   DLG_INVAL,        /* invalid value */
   DLG_FOCUS,        /* can't set focus */
   DLG_NOHELP,       /* no help file */
   DLG_NOTAG,        /* no such help tag */
   DLG_MAXERR
};


/* ------------------------------------------------------------------------

   This will map most function names
   Required by m_scan.c

------------------------------------------------------------------------ */

#ifdef DLG_COMPATIBLE_OBSOLETE

#define dlg_map_classname(classname, classrec) \
     dm_dlg_map_classname(classname, classrec)

#define dlg_new(path, classrec) \
     dm_dlg_new(path, classrec)

#define dlg_del(path) \
     dm_dlg_del(path)

#define dlg_get(path, val) \
     dm_dlg_get(path, val)

#define dlg_set( path, val) \
     dm_dlg_set(path, val)

#define dlg_load(fname, p_msg, p_line_no) \
     dm_dlg_load(fname, p_msg, p_line_no)

#define dlg_draw(path) \
     dm_dlg_draw(path)

#define dlg_do(path, options, exit_rule, exit_path) \
     dm_dlg_do(path, options, exit_rule, exit_path)

#define dlg_stop() \
     dm_dlg_stop()

#define dlg_force_redraw( mode ) 0

#endif   /* DLG_COMPATIBLE */

#endif   /* DLGAPI_INCLUDED */
