/*------------------------------------------------------------------------
   (C) COPYRIGHT HEWLETT-PACKARD GmbH 1994-1996
   
   Project:  HP ELOQUENCE
   Module:   dmdrv
   File:     dmdrv.h
   Function: common things

   This specifies api provided for dm driver

   Configuration:

      DMDRV_PRIVATE    - defined to access internal data structures

      DMDRV_CLIENT_API - by default, SERVER api definition is visible.

      DLGAPI_INCLUDED  - make client wrapper prototypes visible
                         only meaningfull if DMDRV_CLIENT_API defined

   Edit history:
   
   22.12.93 - creation
   25.01.94 - merge hp-ux/msw
   26.01.94 - DMDRV_POPUP, removed DMDRV_CLASS, load status return
   06.02.94 - calling convention
   20.02.94 - async
   11.11.94 - drv_dlg_rule(), drv_dlg_func(), drv_popup_box()
   09.01.95 - DMDRV_HELP

------------------------------------------------------------------------*/

/* @(#)$Revision$ */


#ifndef DMDRV_INCLUDED
#define DMDRV_INCLUDED

/* ------------------------------------------------------------------------

   API types

------------------------------------------------------------------------ */

#ifndef DLGAPI_TYPES
#define DLGAPI_TYPES

#ifndef DOS

# define FAR    /**/
# define PASCAL /**/
# define CDECL  /**/

typedef char *LPSTR;
typedef const char *LPCSTR;

#else    /* DOS */

#ifndef _INC_WINDOWS

/* <windows.h> must be included before */

# define FAR    __far
# define PASCAL __pascal
# define CDECL  __cdecl

typedef char FAR *LPSTR;
typedef const char FAR *LPCSTR;

#endif   /* _INC_WINDOWS */

#endif   /* DOS */
#endif   /* DMAPI_TYPES */


/* ----------------------------------------------------------------------

      portable types

---------------------------------------------------------------------- */      

#ifndef DMDRV_TYPES_INCLUDED

/*
   Portability note:
   - enum sizes are 4 bytes,
   - alignment should be to 4 byte boundary
*/

typedef signed char schar_t;        /* signed char */
typedef unsigned char uchar_t;      /* unsigned char */
typedef signed char int8_t;         /* signed integer, 8 bit */
typedef unsigned char uint8_t;      /* unsigned integer, 8 bit */
typedef signed short int16_t;       /* signed integer, 16 bit */
typedef unsigned short uint16_t;    /* unsigned integer, 16 bit */
typedef signed long int32_t;        /* signed integer, 32 bit */
typedef unsigned long uint32_t;     /* unsigned integer, 32 bit */

#endif


/* ----------------------------------------------------------------------

      constant values

---------------------------------------------------------------------- */      

typedef enum {
   DMDRV_VOID = 0,   /* bad request */
   DMDRV_STARTUP,    /* server startup */
   DMDRV_STOP,       /* stop driver */
   DMDRV_DEBUG,      /* debug request */
   DMDRV_NEW,        /* create new object */
   DMDRV_DEL,        /* delete object (tree) */
   DMDRV_SET,        /* set object attribute */
   DMDRV_GET,        /* get object attribute */
   DMDRV_LOAD,       /* load dialog file */
   DMDRV_DRAW,       /* draw (refresh) object */
   DMDRV_DO,         /* process dialog events */
   DMDRV_CALL,       /* callback */
   DMDRV_STATUS,     /* status response */
   DMDRV_POPUP,      /* message box */
   DMDRV_RULE,       /* call named rule */
   DMDRV_FUNC,       /* call named function */
   DMDRV_ASYNC,      /* async mode select (tcp/ip only) */
   DMDRV_HELP        /* call help on tag */
} t_DMDRV_CALL;

typedef enum {
   DMDRV_vtVOID = 0,
   DMDRV_vtINT,
   DMDRV_vtSTRING
} t_DMDRV_valtype;


/* ------------------------------------------------------------------------

   We have 2 similar data struct's, describing all data passed
   between processes.
   
   This struct will actually passed between different machines
   and must be binary compatible (client is responsible).
   The other one is a local representation of the protable
   struct, using apropriate data types.

   To be portable in means of alignment, each field should
   have a fixed length /a multiple of) of 4 bytes.
------------------------------------------------------------------------ */

#ifdef DMDRV_PRIVATE          /* private to "ipc" routines ... */

typedef struct {
   int32_t type;                    /* t_DMDRV_valtype         */
   union {
      int32_t iv;                   /*  integer value          */
      int32_t sv;                   /*  string value (ofs)     */
    } u;
} t_cDMDRV_value;

typedef struct {
   uint32_t argsz;                  /* size of argument buffer */
                                    /* this MUST be the 1st    */
                                    /* element in struct       */

   int32_t error;                   /* error number            */
                                    /* if nonzero, the         */
                                    /*  remaining structure    */
                                    /*  is invalid             */

   int32_t call;                    /* integer value, as coded */
                                    /* in t_DMDRV_CALL         */

   union {

      struct {                      /* server startup          */
         int32_t argstr;            /*  arg string             */
      } startup;

      struct {                      /* general status          */
         int32_t iv;                /*  completion code        */
      } status;

      struct {                      /* obj path                */
         int32_t path;              /*  object path (ofs)      */
      } obj;

      struct {                      /* debug                   */
         int32_t level;             /*  debug level            */
      } debug;

      struct {                      /* obj get, status         */
         int32_t iv;                /*  completion code        */
         t_cDMDRV_value val;        /*  attribute value        */
      } st_get;

      struct {                      /* dlg do, status          */
         int32_t iv;                /*  completion code        */
         int32_t rule;              /*  obj rule value         */
         int32_t path;              /*  obj path (ofs)         */
      } st_do;

      struct {                      /* dlg load                */
         int32_t fname;             /* filename (ofs)          */
      } load;

      struct {                      /* dlg load, status        */
         int32_t iv;                /* completion code         */
      } st_load;

      struct {                      /* dlg set                 */
         int32_t path;              /*  object path (obj)      */
         t_cDMDRV_value val;        /*  attribute value        */
      } set;

      struct {                      /* dlg new                 */
         int32_t path;              /*  obj path (ofs)         */
         int32_t classnm;           /*  class name             */
      } create;

      struct {                      /* callback request        */
         int32_t fn;                /*  function name (ofs)    */
      } call;

      struct {                      /* call named rule         */
         int32_t name;              /* rule name               */
         int32_t path;              /* "this" object           */
         int32_t argc;              /* number of arguments     */
         t_cDMDRV_value argv[8];    /* arguments               */
      } rule;

      struct {
         int32_t iv;                /* status                  */
         t_cDMDRV_value rval;       /* return value            */
      } st_rule;

      struct {                      /* call named function     */
         int32_t name;              /* function name           */
         int32_t argc;              /* number of arguments     */
         t_cDMDRV_value argv[8];    /* arguments               */
      } func;

      struct {
         int32_t iv;                /* status                  */
         t_cDMDRV_value rval;       /* return value            */
      } st_func;

      struct {
         int32_t style;             /* popup style   (eq = -1) */
         int32_t btn_style;         /* button style  (not eq)  */
         int32_t parent;            /* parent object (not eq ) */
         int32_t x;                 /* x pos         (only eq) */
         int32_t y;                 /* y pos         (only eq) */
         int32_t title;             /* title string            */
         int32_t text;              /* body text               */
         int32_t btn_cnt;           /* button count  (only eq) */
         int32_t btn[5];            /* button_text   (only eq) */
         int32_t def_btn;           /* default button          */
      } popup;

      struct {
         int32_t iv;                /* return status           */
         int32_t btn;               /* selected button         */
      } st_popup;

      struct {                      /* async ipc               */
         int32_t mode;              /*  0 = no, 1 = yes        */
      } async;

      struct {                      /* async ipc reponse       */
         int32_t mode;              /* server mode             */
      } st_async;
      
      struct {			    /* call help on given tag  */
	 int32_t tag;               /* help tag (string)       */
      } help;
   } u;

} t_cDMDRV;

#endif   /* DMDRV_PRIVATE */


/*------------------------------------------------------------------------

   This is the driver native version of portable data type above.

------------------------------------------------------------------------*/

typedef struct {
   t_DMDRV_valtype type;
   union {
      int32_t iv;
      LPCSTR sv;
   } u;
} t_DMDRV_value;

typedef struct
{
   t_DMDRV_CALL call;               /* describes data format   */

   union {

      struct {                      /* server startup          */
         LPCSTR argstr;             /*  arg string             */
      } startup;

      struct {                      /* general status          */
         int iv;                    /*  completion code        */
      } status;

      struct {                      /* obj path                */
         LPCSTR path;               /* object path             */
      } obj;

      struct {                      /* debug                   */
         int level;                 /* debug level             */
      } debug;

      struct {                      /* obj get, status         */
         int iv;                    /* completion code         */
         t_DMDRV_value val;         /* attribute value         */
      } st_get;

      struct {                      /* dlg do, status          */
         int iv;                    /* completion code         */
         int32_t rule;              /* obj rule value (16 bit?)*/
         LPCSTR path;               /* obj path                */
      } st_do;

      struct {                      /* dlg load                */
         LPCSTR fname;              /* filename                */
      } load;

      struct {                      /* dlg load, status        */
         int iv;                    /* completion code         */
      } st_load;

      struct {                      /* dlg set                 */
         LPCSTR path;               /* object path             */
         t_DMDRV_value val;         /* attribute value         */
      } set;

      struct {                      /* dlg new                 */
         LPCSTR path;               /* obj path                */
         LPCSTR classnm;            /*  class name             */
      } create;

      struct {                      /* callback request        */
         LPCSTR fn;                 /* function name           */
         int argc;
         t_DMDRV_value FAR *argv;
      } call;

      struct {                      /* callback return         */
         int iv;                    /* return status           */
         t_DMDRV_value rval;        /* returned value          */
      } st_call;

      struct {                      /* call named rule         */
         LPCSTR name;               /* rule name               */
         LPCSTR path;               /* "this" object           */
         int argc;                  /* number of arguments     */
         t_DMDRV_value argv[8];     /* arguments               */
      } rule;

      struct {
         int32_t iv;                /* status                  */
         t_DMDRV_value rval;        /* return value            */
      } st_rule;

      struct {                      /* call named function     */
         LPCSTR name;               /* function name           */
         int32_t argc;              /* number of arguments     */
         t_DMDRV_value argv[8];     /* arguments               */
      } func;

      struct {
         int32_t iv;                /* status                  */
         t_DMDRV_value rval;        /* return value            */
      } st_func;

      struct {
         int style;                 /* popup style   (eq = -1) */
         int btn_style;             /* button style  (not eq)  */
         LPCSTR parent;             /* parent object (not eq ) */
         int x;                     /* x pos         (only eq) */
         int y;                     /* y pos         (only eq) */
         LPCSTR title;              /* title string            */
         LPCSTR text;               /* body text               */
         int btn_cnt;               /* button count  (only eq) */
         LPCSTR btn[5];             /* button_text   (only eq) */
         int32_t def_btn;           /* default button          */
      } popup;

      struct {
         int32_t iv;                /* return status           */
         int32_t btn;               /* selected button         */
      } st_popup;

      struct {                      /* async ipc               */
         int mode;                  /*  0 = no, 1 = yes        */
      } async;

      struct {                      /* async ipc reponse       */
         int mode;                  /* server mode             */
      } st_async;

      struct {			    /* call help on given tag  */
	 LPCSTR tag;                /* help tag (string)       */
      } help;
   } u;
} t_DMDRV;


/*------------------------------------------------------------------------

      private

------------------------------------------------------------------------*/

#ifdef DMDRV_PRIVATE

/*
   This is only needed if using unix ipc
*/

#define DMDRV_MAGIC_SZ 8
#define DMDRV_MAGIC    "DLGIPC\0"   /* must be MAGIC_SZ bytes */
#define DMDRV_VERSION  0x0101

typedef struct
{
   /* unix ipc specific */

   uchar_t magic[DMDRV_MAGIC_SZ];   /*  magic constant */
   int32_t version;                 /*  version number */
   int32_t pid;                     /*  parent process id (should be pid_t) */
   uint32_t shmsz;                  /*  shm memory size */

   /* general */

   uint32_t memsz;                  /*  size of dynamic argument buffer */
   int32_t error;                   /*  error number (last receive) */
   uint32_t argsz;                  /*  size of argument buffer (last recv) */
   t_cDMDRV arg;                    /*  fixed arguments */

} t_DMDRV_INFO;

/* alignment is done using this macro */
#define ALIGN(sz) (((sz + 3) / 4) * 4)

#endif   /* DMDRV_PRIVATE */


/* ----------------------------------------------------------------------

      prototypes

---------------------------------------------------------------------- */      

/* 
   These error numbers must be accessible from both,
   client and server process. They deal with problems
   detected during converting data.
*/

#define DRV_ST_MEM_OVFL 610    /* Memory overflow during response */
#define DRV_ST_ERROR    611    /* Bad response format */

   
/*
   G L O B A L S
*/

#ifndef DMDRV_CLIENT_API

#include "dmdrv_sv.h"

#else

#include "dmdrv_cl.h"

#endif

#endif   /* DMDRV_INCLUDED */
