/*------------------------------------------------------------------------
   (C) COPYRIGHT HEWLETT-PACKARD GmbH 1994-1996
   
   Project:  HP ELOQUENCE
   Module:   dmdrv
   File:     dmdrv_clnt.h
   Function: client settings

   loaded from dmdrv.h if DMDRV_CLIENT_API is set

   Configuration:

      DMDRV_PRIVATE    - defined to access internal data structures

      DMDRV_CLIENT_API - by default, SERVER api definition is visible.

      DLGAPI_INCLUDED  - make client wrapper prototypes visible
                         only meaningfull if DMDRV_CLIENT_API defined

   Edit history:
   
   09.01.96 - split

------------------------------------------------------------------------*/

/* @(#)$Revision$ */


/*
   C L I E N T   G L O B A L S
*/

#define MAX_DRV 1             /* at least 1 */
#define MAX_DRV_ARGS   100    /* at least 2 */

#define DRV_CANT_LOAD  31    /* (680) Unable to load DRV */
#define DRV_BAD_SIZE   32    /* (681) Improper DRV memory size */
#define DRV_NOT_LOADED 33    /* (682) DRV not loaded */
#define DRV_MEM_OVFL   34    /* (683) DRV memory overflow */
#define DRV_DIED       35    /* (684) DRV process not found */
#define DRV_ERROR      36    /* (685) DRV Return area destroyed */

extern int FAR PASCAL drv_debug_flag;

extern int FAR PASCAL drv_setup( void );
extern void FAR PASCAL drv_reset( void );
extern int FAR PASCAL drv_load(LPCSTR id, LPCSTR fname, size_t sz, int argc, LPCSTR argv[]);
extern int FAR PASCAL drv_unload( LPCSTR id );
extern int FAR PASCAL drv_call( LPCSTR id, const t_DMDRV FAR *req, t_DMDRV FAR **res );

#ifdef DLGAPI_INCLUDED                                         /* dlgapi.h */

extern int FAR PASCAL drv_dlg_stop(LPCSTR drv_id );
extern int FAR PASCAL drv_dlg_get( LPCSTR drv_id, LPCSTR path, VALUE FAR *vp );
extern int FAR PASCAL drv_dlg_set( LPCSTR drv_id, LPCSTR path, const VALUE FAR *vp );
extern int FAR PASCAL drv_dlg_do( LPCSTR drv_id, LPCSTR path, int options,
                       int FAR *exit_rule, LPSTR exit_path );
extern int FAR PASCAL drv_dlg_draw( LPCSTR drv_id, LPCSTR path );
extern int FAR PASCAL drv_dlg_load( LPCSTR drv_id,
                         LPCSTR fname, LPCSTR *msg, int FAR *line_no );
extern int FAR PASCAL drv_dlg_new( LPCSTR drv_id, LPCSTR path, LPCSTR classname );
extern int FAR PASCAL drv_dlg_del( LPCSTR drv_id, LPCSTR path );

extern int FAR PASCAL drv_dlg_rule( LPCSTR drv_id, LPCSTR name, LPCSTR path,
               int argc, const VALUE FAR *argv, VALUE FAR *rvp );
extern int FAR PASCAL drv_dlg_func( LPCSTR drv_id, LPCSTR name,
               int argc, const VALUE FAR *argv, VALUE FAR *rvp );
extern int FAR PASCAL drv_popup_box( LPCSTR drv_id,
               int x, int y, LPCSTR title, LPCSTR text,
               int btn_cnt, LPCSTR btn[], int def_btn, int FAR *rtn_btn );
extern int FAR PASCAL drv_dlg_help( LPCSTR drv_id, LPCSTR tag );

#endif   /* DLGAPI_INCLUDED */


#ifdef DMDRV_PRIVATE

/* global data structure */
extern t_DMDRV_INFO FAR *dmdrv_info;

/* internal compatibility loader */
extern int FAR PASCAL 
drv_do_load( LPCSTR drv_id, LPCSTR fname, LPCSTR *p_msg, int FAR *p_line_no);

/* message routines */
extern void FAR CDECL dmdrv_error(LPCSTR fmt, ...);
extern void FAR CDECL dmdrv_debug(int level, LPCSTR fmt, ...);

/* ipc routines */
extern void FAR PASCAL ipc_cleanup( void );
extern int FAR PASCAL ipc_setup( LPCSTR host, LPCSTR service );
extern int FAR PASCAL ipc_send( void );
extern int FAR PASCAL ipc_schedule( void );

#endif   /* DMDRV_PRIVATE */
