/*------------------------------------------------------------------------
   (C) COPYRIGHT HEWLETT-PACKARD GmbH 1993
   
   Project:  HP ELOQUENCE
   Module:   dlg / dm
   File:     helpview.c
   Function: using vue 3.0 helpview

   Edit history:
   07.01.94 - creation

------------------------------------------------------------------------*/

static char RcsId[] =
   "$Header$";


/* ----------------------------------------------------------------------
      includes
---------------------------------------------------------------------- */

#ifdef DOS
# include <windows.h>
# include <toolhelp.h>
#else
# include <unistd.h>
# include <sys/types.h>
# include <sys/wait.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define DMDRV_PRIVATE
#include "dmdrv.h"

#include <IDMuser.h>   
#include "dlgapi.h"
#include "server.h"

extern const DM_ID FAR *const EqDialogID;

#define FAILED(code,msg) dm_error_handler(__FILE__,__LINE__,code,msg)

#ifdef DOS
/* This will include the DDE management */
# include "ddeclnt.h"
/* This will define the .INI file */
# define INI_FILE "eloq.ini"
#endif

static boolean do_HelpOnTag(DM_ID objID, LPCSTR tag);
static boolean OpenURL(LPCSTR url, LPCSTR path, LPCSTR tag, boolean reload);


/* ----------------------------------------------------------------------

      HelpOnObject

---------------------------------------------------------------------- */

boolean DML_default DM_ENTRY HelpOnObject(DM_ID objID)
{
   DM_Value data;

   dmdrv_debug(2, "HelpOnObject(obj=%ld)", objID);

   while (!DM_GetValue(objID, AT_help, 0, &data, DMF_GetMasterString))
   {   
      /* object does not have .help attribute: get parent */
      if(!DM_GetValue(objID, AT_parent, 0, &data, 0))
      {
         /* no parent: empty tag */
         dmdrv_debug(2, "missing .help attribute, no parent: empty tag");
         return do_HelpOnTag(objID, "");
      }
      objID = data.value.id;
      dmdrv_debug(2, "missing .help attribute, trying parentID=%ld", objID);
   }

   return do_HelpOnTag(objID, data.value.string);
}


/* ----------------------------------------------------------------------

      HelpOnTag

---------------------------------------------------------------------- */

boolean DML_default DM_ENTRY HelpOnTag(DM_ID objID, LPCSTR tag)
{
   dmdrv_debug(2, "HelpOnTag(obj=%ld, tag=%s)", objID, tag);
   return do_HelpOnTag(objID, tag);
}

static boolean do_HelpOnTag(DM_ID objID, LPCSTR tag)
{
   DM_ID dlgID, varID;
   DM_Value data;

   if( !objID )
      dlgID = *EqDialogID;
   else {
      if(!DM_GetValue( objID, AT_dialog, 0, &data, 0 )) {
         (void) FAILED(0, "Unable to obtain dialog ID");
         return(FALSE);
      }
      dlgID = data.value.id;
   }

   varID = DM_DialogPathToID( dlgID, 0, "EqHelpPath" );
   if( !varID ) {
      (void) FAILED(0, "Unable to obtain EqHelpPath variable ID");
      return FALSE;
   }
   
   if(!DM_GetValue( varID, AT_value, 0, &data,
                    DMF_GetMasterString | DMF_DontFreeLastStrings )) {
      (void) FAILED(0, "Unable to obtain EqHelpPath variable value");
      return FALSE;
   }

   return OpenURL(config.help_base_url, data.value.string, tag, 0);
}


/* ----------------------------------------------------------------------

      HelpViewFile / HelpManPage

---------------------------------------------------------------------- */

boolean DML_default DM_ENTRY HelpViewFile(LPCSTR path)
{
   dmdrv_debug(2, "HelpViewFile(path=%s)", path);
   return OpenURL(config.file_base_url, path, "", 1);
}

boolean DML_default DM_ENTRY HelpManPage(LPCSTR path)
{
   dmdrv_debug(2, "HelpManPage(path=%s)", path);
   return OpenURL(config.man_base_url, path, "", 0);
}


#ifdef DOS
/* ----------------------------------------------------------------------

      Windows: perform DDE transaction

---------------------------------------------------------------------- */

static boolean do_DDE(char *module_p, char *topic_p,
                      char *action_p, char *arg_p, char *value_p,
                      char *result_p, int result_max)
{
   HWND hwndClientDDE, hwndServerDDE;
   int retry, inDDE, getData;
   UINT hInst;
   TASKENTRY te;
   char tmp[256];

   dmdrv_debug(2, "do_DDE(module=%s, topic=%s, action=%s, arg=%s, value=%s)",
                  module_p, topic_p, action_p, arg_p, value_p);
   inDDE = FALSE;
         
   // initiate DDE connversation
   InitAckTimeOut(0);
   for (retry = 0;; retry++)
   {
      hwndClientDDE = SendInitiate(module_p, topic_p);
      if (hwndClientDDE == NULL)
      {
         strcpy(tmp, "Could not initiate DDE");
         dmdrv_debug(1, "do_DDE() failed: %s", tmp);
         MessageBox(NULL, tmp,
                    "DDE failed", MB_TASKMODAL | MB_ICONSTOP | MB_OK);
         goto failed;
      }
      inDDE = TRUE;
         
      // get DDE server window handle
      hwndServerDDE = GetHwndServerDDE(hwndClientDDE);
      if (hwndServerDDE != NULL)
         break;
               
      // DDE server not found
      //
      DestroyWindow(hwndClientDDE);
      inDDE = FALSE;

      // on first try get module file name from INI file if any
      // if specified: load module
      if (retry == 0)
      {
         GetPrivateProfileString("modules", module_p, "", 
                                 tmp, sizeof(tmp), INI_FILE);
         if (*tmp != '\0')
            hInst = WinExec(tmp, SW_SHOW);
      }

      // error on second try or if module file name is not
      // specified in INI file
      //
      if (retry > 0 || *tmp == '\0' || hInst < HINSTANCE_ERROR)
      {
         sprintf(tmp, "Module \'%s\' is not active"
                      " and I don't know how to start it", module_p);
         dmdrv_debug(1, "do_DDE() failed: %s", tmp);
         MessageBox(NULL, tmp,
                    "DDE failed", MB_TASKMODAL | MB_ICONSTOP | MB_OK);
         goto failed;
      }

      // lookup module task ID
      //
      te.dwSize = (DWORD)sizeof(TASKENTRY);
      if (TaskFirst(&te))
      {
         do {
            if (te.hInst == (HINSTANCE)hInst)
            {
               // found: force execution
               DirectedYield(te.hTask);
               break;
            }
         } while (TaskNext(&te));
      }
   }         

   // do desired action
   //
   getData = FALSE;
   if (!stricmp(action_p, "execute"))
   {
      SendExecute(hwndClientDDE, hwndServerDDE, arg_p);
   }
   else if (!stricmp(action_p, "request"))
   {
      SendRequest(hwndClientDDE, hwndServerDDE, arg_p);
      getData = TRUE;
   }
   else if (!stricmp(action_p, "poke"))
   {
      SendPoke(hwndClientDDE, hwndServerDDE, arg_p, value_p);
   }
   else {
      sprintf(tmp, "Unknown DDE Action \'%s\'", action_p);
      dmdrv_debug(1, "do_DDE() failed: %s", tmp);
      MessageBox(NULL, tmp,
                 "DDE failed", MB_TASKMODAL | MB_ICONSTOP | MB_OK);
      goto failed;
   }

   if (WaitForAck(hwndClientDDE) != ACK)
   {
      strcpy(tmp, "DDE server did not acknowledge");
      dmdrv_debug(1, "do_DDE() failed: %s", tmp);
      MessageBox(NULL, tmp,
                 "DDE failed", MB_TASKMODAL | MB_ICONSTOP | MB_OK);
      goto failed;               
   }

   if (getData)
   {
      if (!GetConvItemValue(hwndClientDDE, arg_p, result_p, result_max))
      {
         strcpy(tmp, "DDE data not available");
         dmdrv_debug(1, "do_DDE() failed: %s", tmp);
         MessageBox(NULL, tmp,
                    "DDE failed", MB_TASKMODAL | MB_ICONSTOP | MB_OK);
         goto failed;
      }
      if (result_p)
         dmdrv_debug(2, "do_DDE() result=[%s]", result_p);
   }

   // close DDE conversation(s)
   TerminateConversations();
   return TRUE;

failed:
   if (inDDE)
      TerminateConversations();
   return FALSE;
}
#endif /* DOS */


/* ----------------------------------------------------------------------

      Windows: launch netscape as help viewer via DDE

---------------------------------------------------------------------- */

static boolean OpenURL(LPCSTR url, LPCSTR path, LPCSTR tag, boolean reload)
{
   char tmp[1024];
   int rc;

   dmdrv_debug(2, "OpenURL(url=%s, path=%s, tag=%s)", url, path, tag);

#ifdef DOS

   strcpy(tmp, "\"");
   strcat(tmp, url);
   strcat(tmp, path);
   strcat(tmp, tag);
   strcat(tmp, "\",,0xFFFFFFFF,");
   strcat(tmp, reload ? "0x3" : "0x0");

   // launch NETSCAPE, open URL
   rc = do_DDE("netscape", "WWW_OpenURL", "REQUEST", tmp, "", NULL, 0);
   if (!rc)
      return FALSE;

   // put NETSCAPE on top
   rc = do_DDE("netscape", "WWW_Activate", "REQUEST",
                           "0xFFFFFFFF,0x0", "", NULL, 0);
   if (!rc)
      return FALSE;

#else

   if( access(config.netscape, X_OK) == -1 )
   {
      dmdrv_debug(0, "%s: not executable", config.netscape);
      return FALSE;
   }

   strcpy(tmp, config.netscape);
   strcat(tmp, " -remote \"OpenURL(");
   strcat(tmp, url);
   strcat(tmp, path);
   strcat(tmp, tag);
   strcat(tmp, ")\"");

   dmdrv_debug(2, "system(%s)", tmp);
   rc = system(tmp);
   if (!WIFEXITED(rc) || WEXITSTATUS(rc))
      dmdrv_debug(2, "Netscape: rc=0x%x", rc);

   if (WIFEXITED(rc) && WEXITSTATUS(rc) == 1)
   {
      /* netscape not running: start it */
      strcpy(tmp, config.netscape);
      strcat(tmp, " ");
      strcat(tmp, url);
      strcat(tmp, path);
      strcat(tmp, tag);
      strcat(tmp, " &");

      dmdrv_debug(2, "starting Netscape...");
      dmdrv_debug(2, "system(%s)", tmp);
      rc = system(tmp);
      if (!WIFEXITED(rc) || WEXITSTATUS(rc))
         dmdrv_debug(2, "Netscape: rc=0x%x", rc);
   }
   else if (reload && WIFEXITED(rc) && WEXITSTATUS(rc) == 0)
   {
      strcpy(tmp, config.netscape);
      strcat(tmp, " -remote reload");
   
      dmdrv_debug(2, "system(%s)", tmp);
      rc = system(tmp);
      if (!WIFEXITED(rc) || WEXITSTATUS(rc))
         dmdrv_debug(2, "Netscape: rc=0x%x", rc);
   }

#endif

   return TRUE;
}

